@@ -1,9 +1,9 @@
 /* vi: set sw=4 ts=4: */
-/* Copyright 2001 Glenn McGrath.
+/*
+ * Copyright 2001 Glenn McGrath.
  *
  * Licensed under GPLv2 or later, see file LICENSE in this source tree.
  */
-
 #include "libbb.h"
 #include "bb_archive.h"
 #include "ar.h"
@@ -127,8 +127,10 @@ char FAST_FUNC get_header_ar(archive_handle_t *archive_handle)
 		archive_handle->action_header(typed);
 #if ENABLE_DPKG || ENABLE_DPKG_DEB
 		if (archive_handle->dpkg__sub_archive) {
-			while (archive_handle->dpkg__action_data_subarchive(archive_handle->dpkg__sub_archive) == EXIT_SUCCESS)
+			struct archive_handle_t *sa = archive_handle->dpkg__sub_archive;
+			while (archive_handle->dpkg__action_data_subarchive(sa) == EXIT_SUCCESS)
 				continue;
+			create_symlinks_from_list(sa->symlink_placeholders);
 		} else
 #endif
 			archive_handle->action_data(archive_handle);
